@@ -33,6 +33,79 @@ class FormType(Type):
         return value
 
 
+class NameField(models.CharField):
+    """Almost but not quite a slug field. We only allow snake_case_names."""
+
+    def __init__(self, **kwargs):
+        kwargs.setdefault("verbose_name", _("name"))
+        kwargs.setdefault("max_length", 50)
+        kwargs.setdefault(
+            "validators",
+            [
+                validators.RegexValidator(
+                    r"^[a-z0-9_]+$",
+                    message=_(
+                        "Enter a value consisting only of lowercase letters,"
+                        " numbers and the underscore."
+                    ),
+                ),
+            ],
+        )
+        kwargs.setdefault(
+            "help_text",
+            _(
+                "Data is saved using this name. Changing it may result in data loss."
+                " This field only allows a-z, 0-9 and _ as characters."
+            ),
+        )
+        super().__init__(**kwargs)
+
+    def deconstruct(self):
+        name, path, args, kwargs = super().deconstruct()
+        return name, "django.db.models.CharField", args, kwargs
+
+
+class FormFieldBase(models.Model):
+    """
+    Form field plugins must inherit this model if using the
+    ``get_formfields_union`` helper for configured form validation.
+    """
+
+    name = NameField()
+
+    class Meta:
+        abstract = True
+
+    def get_fields(self, **kwargs):
+        """
+        Return a dictionary of form fields
+
+        The keys are form field names (prefixed by ``self.name``), the values
+        form field instances.
+        """
+        raise NotImplementedError(
+            f"{self._meta.label_lower} needs a get_fields implementation"
+        )
+
+    def get_initial(self):
+        """The default implementation returns no initial values"""
+        return {}
+
+    def get_loaders(self):
+        """
+        Return a list of loaders
+
+        Loaders are callables which receive the serialized form data and
+        return a dictionary of the following shape::
+
+            {"name": ..., "label": ..., "value": ...}
+
+        """
+        raise NotImplementedError(
+            f"{self._meta.label_lower} needs a get_loaders implementation"
+        )
+
+
 class ConfiguredForm(models.Model):
     name = models.CharField(_("name"), max_length=1000)
     form_type = ChoicesCharField(_("form type"), max_length=100)
@@ -63,10 +136,12 @@ class ConfiguredForm(models.Model):
             types = {type.key: type for type in sender.FORMS}
             sender.type = property(lambda self: types.get(self.form_type))
 
-    def get_formfields_union(self, *, plugins, values=["name"]):
+    def get_formfields_union(
+        self, *, plugins, values=["name"], field_plugin_bases=(FormFieldBase,)
+    ):
         qs = None
         for plugin in plugins:
-            if not issubclass(plugin, FormField):
+            if not issubclass(plugin, field_plugin_bases):
                 continue
             plugin_qs = plugin.objects.filter(parent=self).values_list(
                 *values, flat=len(values) == 1
@@ -81,39 +156,8 @@ class ConfiguredForm(models.Model):
 signals.class_prepared.connect(ConfiguredForm.fill_form_choices)
 
 
-class NameField(models.CharField):
-    def __init__(self, **kwargs):
-        kwargs.setdefault("verbose_name", _("name"))
-        kwargs.setdefault("max_length", 50)
-        kwargs.setdefault(
-            "validators",
-            [
-                validators.RegexValidator(
-                    r"^[a-z0-9_]+$",
-                    message=_(
-                        "Enter a value consisting only of lowercase letters,"
-                        " numbers and the underscore."
-                    ),
-                ),
-            ],
-        )
-        kwargs.setdefault(
-            "help_text",
-            _(
-                "Data is saved using this name. Changing it may result in data loss."
-                " This field only allows a-z, 0-9 and _ as characters."
-            ),
-        )
-        super().__init__(**kwargs)
-
-    def deconstruct(self):
-        name, path, args, kwargs = super().deconstruct()
-        return name, "django.db.models.CharField", args, kwargs
-
-
-class FormField(models.Model):
+class FormField(FormFieldBase):
     label = models.CharField(_("label"), max_length=1000)
-    name = NameField()
     is_required = models.BooleanField(_("is required"), default=True)
     help_text = models.CharField(
         _("help text"),
@@ -137,8 +181,8 @@ class FormField(models.Model):
         return [partial(simple_loader, label=self.label, name=self.name)]
 
 
-def simple_loader(data, *, label, name):
-    return {"label": label, "name": name, "value": data.get(name)}
+def simple_loader(data, *, name, label):
+    return {"name": name, "label": label, "value": data.get(name)}
 
 
 class SimpleFieldBase(FormField):
